// Source file for the particle system



// Include files 

#ifdef _WIN32
#  include <windows.h>
#  define M_PI 3.1415925
#else
#  include <sys/time.h>
#endif
#include "R3/R3.h"
#include "R3Scene.h"
#include "particle.h"



// Particle system rendering parameters

int particle_render_type = RENDER_POINT_PARTICLES;
bool particle_trails = false;



// Particle system simulation parameters

int particle_integration_type = EULER_INTEGRATION;
int particle_drag_constant = 1;
double particle_gravity_acceleration = 9.80665; // in meters/second^2
R3Vector particle_gravity_direction = R3negy_vector;



// Local function declaration

static double GetTime(void);



////////////////////////////////////////////////////////////
// Particle system code
////////////////////////////////////////////////////////////

void UpdateParticles(R3Scene *scene, double current_time)
{
  // Update position (and possibly other parameters) for every particle

  // FILL IN IMPLEMENTATION HERE
}



void GenerateParticles(R3Scene *scene, double current_time)
{
  // Generate new particles for every source

  // FILL IN IMPLEMENTATION HERE
}



void RenderParticles(R3Scene *scene, double current_time)
{
  // Draw every particle with its current parameters
  if (particle_render_type == RENDER_POINT_PARTICLES) {
    // Draw points
    glDisable(GL_LIGHTING);
    R3Rgb background = scene->background;
    glColor3d(1 - background[0], 1 - background[1], 1 - background[2]);
    glPointSize(5);
    glBegin(GL_POINTS);
    for (int i = 0; i < scene->NParticles(); i++) {
      R3Particle *particle = scene->Particle(i);
      const R3Point& position = particle->position;
      glVertex3f(position[0], position[1], position[2]);
    }   
    glEnd();
  }
  else {
    // FILL IN IMPLEMENTATION HERE 
  }
}



void DrawParticles(R3Scene *scene)
{
  // Get current time (in seconds) since start of execution
  double current_time = GetTime();
  // printf("%g\n", current_time);

  // Update particles
  UpdateParticles(scene, current_time);

  // Generate new particles
  GenerateParticles(scene, current_time);

  // Render particles
  RenderParticles(scene, current_time);
}



////////////////////////////////////////////////////////////
// Timer code
////////////////////////////////////////////////////////////


static double GetTime(void)
{
#ifdef _WIN32
  // Return number of seconds since start of execution
  static int first = 1;
  static LARGE_INTEGER timefreq;
  static LARGE_INTEGER start_timevalue;

  // Check if this is the first time
  if (first) {
    // Initialize first time
    QueryPerformanceFrequency(&timefreq);
    QueryPerformanceCounter(&start_timevalue);
    first = 0;
    return 0;
  }
  else {
    // Return time since start
    LARGE_INTEGER current_timevalue;
    QueryPerformanceCounter(&current_timevalue);
    return ((double) current_timevalue.QuadPart - 
            (double) start_timevalue.QuadPart) / 
            (double) timefreq.QuadPart;
  }
#else
  // Return number of seconds since start of execution
  static int first = 1;
  static struct timeval start_timevalue;

  // Check if this is the first time
  if (first) {
    // Initialize first time
    gettimeofday(&start_timevalue, NULL);
    first = 0;
    return 0;
  }
  else {
    // Return time since start
    struct timeval current_timevalue;
    gettimeofday(&current_timevalue, NULL);
    int secs = current_timevalue.tv_sec - start_timevalue.tv_sec;
    int usecs = current_timevalue.tv_usec - start_timevalue.tv_usec;
    return (double) (secs + 1.0E-6F * usecs);
  }
#endif
}



